<?php

namespace Teknisa\Libs\Service;

use Teknisa\Libs\Util\ActiveDirectory;
use Teknisa\Libs\Util\InstanceProvider;
use Teknisa\Libs\Util\Messages;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Repositories;
use Zeedhi\Framework\Session\Session;

class Login {
    /** @var Session $sessionManager */
    protected $sessionManager;
    protected $loginUrl;
    protected $loginClientSecret;

    /** @var Utilities */
    private $utilities;
    /** @var Environment */
    private $environment;
    /** @var ActiveDirectory */
    private $activeDirectory;
    /** @var Repositories\UserData */
    private $userDataRepository;

    /**
     * @param $sessionManager
     * @param Utilities $utilities
     * @param Environment $environment
     * @param ActiveDirectory $activeDirectory
     * @param Repositories\UserData $userDataRepository
     */
    public function __construct($sessionManager, Utilities $utilities, Environment $environment, ActiveDirectory $activeDirectory,
        Repositories\UserData $userDataRepository){
        $this->sessionManager  = $sessionManager;
        $this->utilities       = $utilities;
        $this->environment     = $environment;
        $this->activeDirectory = $activeDirectory;

        $config = json_decode(file_get_contents(__DIR__ . '/../../../../config/config.json'), true);
        $this->loginUrl          = $config['loginUrl'];
        $this->loginClientSecret = $config['loginClientSecret'];
        $this->userDataRepository = $userDataRepository;
    }

    /**
     * @return null|string
     */
    public function getBHash() {
        return $this->sessionManager->get('CURRENT_DBNAME');
    }

    /**
     * @param $token
     * @param $versionFile
     * @param $testDataBase
     * @param $productId
     * @param $isLogged
     * @throws \Exception
     * @return array
     */
    public function restoreSession($token, $versionFile, $testDataBase, $productId, $isLogged) {
        $dataSet = array(
            "requestType" => "FilterData",
            "filter" => array (
                array (
                    'name' => 'BHASH',
                    'operator' => '=',
                    'value' => $this->getBHash()
                ), array (
                    'name' => 'PRODUCT_ID',
                    'operator' => '=',
                    'value' => $productId
                )
            )
        );

        $userData = $this->utilities->callWebService($token, '/lm_requestUserData',
            $dataSet, 'userData', $this->loginClientSecret, $this->loginUrl, $testDataBase);

        if(!empty($userData)) {
            $this->environment->setLoginTokenInSession($token);
            $this->initSession($userData);
            $userData['VERSION'] = $this->getVersionFile($versionFile);
            $userData['SESSION_EXPIRED'] = $this->environment->getSessionExpired();
            $userData['IS_PENDENT_MIGRATION'] = $this->checkPendentMigrations($productId, $userData['ORGANIZATION_ID']);
            if(!$isLogged) {
                $this->userDataRepository->updateUserLastAccess($userData['USER_ID'], $userData['ORGANIZATION_ID']);
            }
            
            return $userData;
        } else {
            throw new \Exception(Messages::ERR_REQUEST_USER_DATA);
        }
    }

    private function checkPendentMigrations($productId, $nrOrg) {
        $isPendentMigration = false;
        $migrationInfo = array();

        try {
            $migrationInfo = InstanceProvider::getMigrationService()->getPendentMigrations($productId, $nrOrg);
        } catch (\Exception $e) {}

        if(!empty($migrationInfo) && !isset($migrationInfo['ERROR'])) {
            foreach ($migrationInfo as $migration) {
                if ($migration['MIG_LAST_FILE_NUMBER'] && $migration['MIG_LAST_FILE_NUMBER'] < $migration['MIG_HIGHER_FILE_NUMBER']) {
                    $isPendentMigration = true;
                    break;
                }
            }
        }
        return $isPendentMigration;
    }

    /**
     * @throws \Exception
     */
    public function logout() {
        $this->sessionManager->start();
        $dataSet = array (
            "requestType" => "FilterData",
            "filter" => array (
                array (
                    'name' => 'BHASH',
                    'operator' => '=',
                    'value' => $this->getBHash()
                )
            )
        );
        $this->utilities->callWebService($this->environment->getLoginTokenInSession(), '/lm_logout',
            $dataSet, 'logout', $this->loginClientSecret, $this->loginUrl);
        $this->sessionManager->destroy();
    }

    /**
     * @param $user
     * @param $password
     * @param $url
     * @return array|null
     */
    public function validateUser($user, $password, $url) {
        $dataSet = array (
            "requestType" => "FilterData",
            "filter" => array (
                array (
                    'name' => 'USER',
                    'operator' => '=',
                    'value' => $user
                ), array (
                    'name' => 'PASSWORD',
                    'operator' => '=',
                    'value' => $password
                ), array (
                    'name' => 'URL',
                    'operator' => '=',
                    'value' => $url
                )
            )
        );

        $userValidation = $this->utilities->callWebService($this->environment->getLoginTokenInSession(),
            '/lm_validateUser', $dataSet, 'userValidation', $this->loginClientSecret, $this->loginUrl);

        return $userValidation;
    }

    /**
     * @param $userData
     */
    private function initSession($userData)
    {
        $this->sessionManager->start();

        $this->sessionManager->set('USER_ID',         $userData['USER_ID']);
        $this->sessionManager->set('USER_NAME',       $userData['USER_NAME']);
        $this->sessionManager->set('ORGANIZATION_ID', $userData['ORGANIZATION_ID']);
        $this->sessionManager->set('TRADING_NAME',    $userData['TRADING_NAME']);
        $this->sessionManager->set('EMAIL',           $userData['EMAIL']);
        $this->sessionManager->set('ACCESS_PROFILE',  $userData['ACCESS_PROFILE']);

        if(!empty($userData['IP'])) {
            $this->sessionManager->set('IP', $userData['IP']);
            $this->sessionManager->set('PORT', $userData['PORT']);
        }

        $this->sessionManager->set('CDOPERADOR',  $userData['USER_ID']);
        $this->sessionManager->set('NMOPERADOR',  $userData['USER_NAME']);
        $this->sessionManager->set('NRORG',       $userData['ORGANIZATION_ID']);
        $this->sessionManager->set('NMORG',       $userData['TRADING_NAME']);
        $this->sessionManager->set('DSEMAILOPER', $userData['EMAIL']);
    }

    /**
     * @param $file
     * @return null|array
     */
    private function getVersionFile($file) {
        if(file_exists($file)) {
            return json_decode(file_get_contents($file), true);
        }
        return false;
    }

    /**
     * @param $user
     * @param $accessProfile
     * @param $host
     * @param $port
     * @param $otherSessionValues
     * @return array
     */
    public function updateSession($user, $accessProfile, $host, $port, $otherSessionValues) {
        $dataSet = array (
            "requestType" => "FilterData",
            "filter" => array (
                array (
                    'name' => 'USER',
                    'operator' => '=',
                    'value' => $user
                ), array (
                    'name' => 'ACCESS_PROFILE',
                    'operator' => '=',
                    'value' => $accessProfile
                ), array (
                    'name' => 'IP',
                    'operator' => '=',
                    'value' => $host
                ), array (
                    'name' => 'PORT',
                    'operator' => '=',
                    'value' => $port
                ), array (
                    'name' => 'OTHER_SESSION_VALUES',
                    'operator' => '=',
                    'value' => $otherSessionValues
                ), array (
                    'name' => 'BHASH',
                    'operator' => '=',
                    'value' => $this->getBHash()
                )
            )
        );

        $userSession = $this->utilities->callWebService($this->environment->getLoginTokenInSession(),
            '/lm_updateSession', $dataSet, 'sessionData', $this->loginClientSecret, $this->loginUrl);

        return $userSession;
    }

    public function getCurrentIp() {
        return Utilities::getCurrentIp();
    }

    public function validateExpiredSession($timeout) {
        $lastRequestTime = $this->sessionManager->get('LAST_REQUEST_TIME');
        $sessionInfo = array('IS_EXPIRED' => false);

        if($lastRequestTime) {
            $date = new \DateTime();
            $actualRequestTime = $date->getTimestamp();
            $diffTime = $actualRequestTime - $lastRequestTime;

            if($diffTime >= ($timeout / 1000)) {
                $sessionInfo['IS_EXPIRED'] = true;
                $this->environment->setSessionExpired(true);
            } else {
                $sessionInfo['TIMEOUT'] = $timeout - ($diffTime * 1000);
            }
        }

        return $sessionInfo;
    }
}